package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysDashboardService;
import cn.estsh.i3plus.platform.common.tool.SerializeTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.ResultBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.web.session.mgt.DefaultWebSessionManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description : Dashboard服务
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2018-12-10 17:04
 * @Modify:
 **/
@RestController
@Api(description = "Dashboard服务")
@RequestMapping(PlatformConstWords.BASE_URL + "/dashboard")
public class SysDashboardController extends CoreBaseController {

    @Autowired
    private ISysDashboardService sysDashboardService;

    @Resource(name="redisCore")
    private ImppRedis redisCore;

    @Resource(name="imppSessionManager")
    private DefaultWebSessionManager webSessionManager;

    @GetMapping("/get")
    @ApiOperation(value = "获取dashboard数据")
    public ResultBean getDashboardData(){
        try {
            Object ds = redisCore.getObject(PlatformConstWords.DASHBOARD_REDIS_KEY + new Date().getTime());
            if(ds == null){
                // 前10分钟的日期时间
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                Calendar cal = Calendar.getInstance();
                cal.setTime(new Date());

                String endTime = TimeTool.getNowTime(true);
                cal.add(Calendar.MINUTE,-10);
                String startTime = sdf.format(cal.getTime());

                Map dashboardData = new HashMap<>();
                dashboardData.put("orgNum", sysDashboardService.getSysOrganizeCount());
                dashboardData.put("deptNum",sysDashboardService.getSysDepartmentCount());
                dashboardData.put("onlineUser",webSessionManager.getSessionDAO().getActiveSessions().size());
                dashboardData.put("logOperate",sysDashboardService.findNewSysLogOperateSize(10));
                dashboardData.put("logException",sysDashboardService.findNewSysLogExceptionSize(10));
                dashboardData.put("avgExecuteTime",sysDashboardService.querySysLogSystemAvgExecuteTime(startTime,endTime));
                dashboardData.put("levelLogNum",sysDashboardService.querySysLogSystemByLevel(startTime,endTime));

                redisCore.putObject(PlatformConstWords.DASHBOARD_REDIS_KEY + new Date().getTime(),
                        SerializeTool.serialization(dashboardData),5);

                return ResultBean.success("添加成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                        .setResultMap(dashboardData);
            }else{
                return ResultBean.success("添加成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                        .setResultMap((Map<Object, Object>) SerializeTool.deserialization(ds.toString()));
            }
        } catch (
                ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
